//
// This code is part of https://developer.mescius.com/document-solutions/dot-net-imaging-api/demos.
// Copyright (c) MESCIUS inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using System.Linq;
using System.Numerics;
using GrapeCity.Documents.Drawing;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Imaging;
using GCTEXT = GrapeCity.Documents.Text;
using GCDRAW = GrapeCity.Documents.Drawing;

namespace DsImagingWeb.Demos
{
    // This sample demonstrates how convert full color RGB
    // to 4 BPP and 8 BPP indexed images.
    public class Indexing
    {
        public GcBitmap GenerateImage(Size pixelSize, float dpi, bool opaque, string[] sampleParams = null)
        {
            var bmp = new GcBitmap(pixelSize.Width, pixelSize.Height, opaque, dpi, dpi);
            using (var origBmp = new GcBitmap())
            {
                // Load a sample photo:
                var imagePath = Path.Combine("Resources", "Images", "maple.jpg");
                using (var stm = new FileStream(imagePath, FileMode.Open, FileAccess.Read, FileShare.Read, 4096, FileOptions.RandomAccess))
                    origBmp.Load(stm);

                origBmp.SetAlphaTo255();
                origBmp.Opaque = opaque;

                // Resize the original photo so we can place 4 samples of it
                // on the resulting bitmap:
                int w = pixelSize.Width / 2;
                int h = pixelSize.Height / 2;
                using (var sizedBmp = origBmp.Resize(w, h, InterpolationMode.Cubic))
                using (var ib4bpp = sizedBmp.ToIndexed4bppBitmap(DitheringMethod.JarvisJudiceNinke))
                using (var ib8bpp = sizedBmp.ToIndexed8bppBitmap(DitheringMethod.JarvisJudiceNinke))
                using (var b4bpp = ib4bpp.ToGcBitmap())
                using (var b8bpp = ib8bpp.ToGcBitmap())
                {
                    // Copy the original and indexed images into 4 quadrants of the resulting bitmap:
                    bmp.BitBlt(sizedBmp, 0, 0);
                    b4bpp.Opaque = opaque;
                    bmp.BitBlt(b4bpp, w, 0);
                    b8bpp.Opaque = opaque;
                    bmp.BitBlt(b8bpp, 0, h);
                    bmp.BitBlt(sizedBmp, w, h);
                }

                // Add borders between the quadrants, and captions for each:
                var lineh = 2;
                using (var g = bmp.CreateGraphics(null))
                {
                    var foreColor = Color.Yellow;
                    var backColor = Color.Blue;
                    var font = GCTEXT.Font.FromFile(Path.Combine("Resources", "Fonts", "FreeMono.ttf"));
                    g.DrawLine(w, 0, w, h * 2, new GCDRAW.Pen(foreColor, lineh * 2));
                    g.DrawLine(0, h, w * 2, h, new GCDRAW.Pen(foreColor, lineh * 2));
                    var tf = new TextFormat() { Font = font, FontSize = 18, ForeColor = foreColor, BackColor = backColor, FontBold = true };
                    g.DrawString(" Original image ", tf, new PointF(0, 0));
                    g.DrawString(" Indexed 4 bits per pixel ", tf, new PointF(w + lineh, 0));
                    g.DrawString(" Indexed 8 bits per pixel ", tf, new PointF(0, h + lineh));
                    g.DrawString(" Original image ", tf, new PointF(w + lineh, h + lineh));
                }
            }
            return bmp;
        }
    }
}
